# List of names which can be equated with each other
# Do not put names that cannot be considered equivalent
# in the same tuple, even if they have the same nickname.
# For instance, John and Jacob could both be shortened to
# Jack, but unless you want John and Jacob to be stand-ins
# for each other, they should not be together.
NICKNAMES = (
    ('Allan', 'Allen', 'Alan', 'Al'),
    ('Allison', 'Alison', 'Ali', 'Aly', 'Allie'),
    ('Andre', u'Andr\00e9'),
    ('Andrew', 'Andy', 'Drew'),
    ('Antonio', 'Anthony', 'Tony', 'Anton'),
    ('Barbara', 'Barb'),
    ('Benjamin', 'Ben'),
    ('Bernard', 'Bernie'),
    ('Calvin', 'Cal'),
    ('Charles', 'Chas', 'Chuck', 'Chucky', 'Charlie'),
    ('Christine', 'Christina', 'Chris'),
    ('Christopher', 'Chris'),
    ('Daniel', 'Dan', 'Danny'),
    ('David', 'Dave'),
    ('Donald', 'Don', 'Donny'),
    ('Douglas', 'Doug'),
    ('Edward', 'Ed', 'Eddie'),
    ('Elizabeth', 'Liz', 'Lizbet', 'Lizbeth', 'Beth', 'Lizzie'),
    ('Francis', 'Frank', 'Frankie'),
    ('Fredrick', 'Frederick', 'Fred', 'Freddy'),
    ('Gerard', 'Gerry', 'Jerry'),
    ('Gerald', 'Gerry', 'Jerry'),
    ('Gregory', 'Greg'),
    ('Harris', 'Harry', 'Harrison'),
    ('Henry', 'Hank'),
    ('Herbert', 'Herb'),
    ('Howard', 'Howie'),
    ('James', 'Jim', 'Jimmy'),
    ('Jerome', 'Jerry'),
    ('John', 'Jon', 'Johnny', 'Jack'),
    ('Joseph', 'Joe'),
    ('Judith', 'Judy'),
    ('Johnathan', 'John'),
    ('Jonathan', 'Jon'),
    ('Katherine', 'Kathy', 'Katie'),
    ('Catherine', 'Cathy', 'Kathy'),
    ('Kathleen', 'Kathy'),
    ('Kenneth', 'Ken', 'Kenny'),
    ('Lawrence', 'Laurence', 'Larry'),
    ('Lewis', 'Louis', 'Lou'),
    ('Matthew', 'Matt'),
    ('Margaret', 'Marge', 'Margie', 'Maggie', 'Meg'),
    ('Martin', 'Marty'),
    ('Melvin', 'Melvyn' ,'Mel'),
    ('Mervyn', 'Merv'),
    ('Michael', 'Mike'),
    ('Mitchell', 'Mitch'),
    ('Nicholas', 'Nick'),
    ('Patricia', 'Pat', 'Patty', 'Pati'),
    ('Patrick', 'Pat'),
    ('Peter', 'Pete'),
    ('Philip', 'Phillip', 'Phil'),
    ('Randall', 'Randy'),
    ('Richard', 'Rick', 'Dick', 'Rich'),
    ('Robert', 'Rob', 'Robby', 'Bobby', 'Bob'),
    ('Steven', 'Stephen', 'Steve'),
    ('Stephanie', 'Steph'),
    ('Stewart', 'Stuart', 'Stu'),
    ('Theodore', 'Ted', 'Teddy'),
    ('Terrance', 'Terry'),
    ('Thomas', 'Tom', 'Thom', 'Tommy'),
    ('William', 'Bill', 'Billy', 'Will', 'Willy'),
    ('Willis', 'Will'),
)
